/**
 * @author Gerald Madlmayr, NFC Research Lab Hagenberg, www.nfc-research.at (original) & Forum Nokia
 * @version 1.1
 *
 * MIDlet that reads the UID of a tag
 */

package com.nokia.example;

// Classes for contactless communication
import javax.microedition.contactless.ContactlessException;
import javax.microedition.contactless.DiscoveryManager;
import javax.microedition.contactless.TargetListener;
import javax.microedition.contactless.TargetProperties;
import javax.microedition.contactless.TargetType;

// Classes for UI
import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;

import javax.microedition.midlet.*;

public class UIDReader
        extends MIDlet
        implements TargetListener, CommandListener {

    // Global variables for the UI
    private Command exitCommand;
    private Form form;

    public UIDReader() {

        // Create the UI
        exitCommand = new Command("Exit", Command.EXIT, 1);
        form = new Form("NFC Research Lab: UIDReader");

        form.addCommand(exitCommand);
        form.setCommandListener(this);
        form.append("Touch tag to read ID");

        // Register TargetListener (this) for NDEF targets
        try {
            DiscoveryManager dm = DiscoveryManager.getInstance();
            dm.addTargetListener(this, TargetType.NDEF_TAG);

        } catch (ContactlessException ce) {
            displayAlert("Unable to register TargetListener: " + ce.toString(), AlertType.ERROR);
        }

    }

    public void startApp() {
        Display.getDisplay(this).setCurrent(form);
    }

    public void pauseApp() {
    }

    public void destroyApp(boolean unconditional) {
    }

    /**
     * Implements the callback function of TargetListener
     * @param targetProperties: Array of targets found by the device
     */
    public void targetDetected(TargetProperties[] targetProperties) {

        // In case no targets are found, exit the method
        if (targetProperties.length == 0) {
            return;
        }

        // Read and display the UID of the first target found
        TargetProperties tmp = targetProperties[0];
        displayAlert("UID read: " + tmp.getUid(), AlertType.INFO);
    }

    /**
     * Implements the callback function of CommandListener
     * @param command: Command given
     * @param displayable: Associated Displayable object
     */
    public void commandAction(Command command, Displayable displayable) {
        if (command == exitCommand) {
            DiscoveryManager dm = DiscoveryManager.getInstance();
            dm.removeTargetListener(this, TargetType.NDEF_TAG);
            destroyApp(false);
            notifyDestroyed();
        }
    }

    private void displayAlert(String error, AlertType type) {
        Alert err = new Alert(form.getTitle(), error, null, type);
        Display.getDisplay(this).setCurrent(err, form);
    }
}